// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

library omnitest.test.calculator;

import 'dart:async';

import 'package:http/http.dart' as http;
import 'package:omnibus/omnibus.dart';

import '../messages/messages.dart' as v0;

/// Calculator module main function
///
/// Binds all calculation handlers to the Omnibus.
void main(Omnibus bus, http.BaseClient httpClient) {
  // Bind calculation handlers.
  bus.bind(new v0.CalculationRequestHandler(calcAddHandler,
      op: v0.CalculationOperator.add));
  bus.bind(new v0.CalculationRequestHandler(calcSubtractHandler,
      op: v0.CalculationOperator.subtract));
  bus.bind(new v0.CalculationRequestHandler(calcMultiplyHandler,
      op: v0.CalculationOperator.multiply));
  bus.bind(new v0.CalculationRequestHandler(calcDivideHandler,
      op: v0.CalculationOperator.divide));
}

Future<v0.CalculationResponse> calcAddHandler(v0.CalculationRequest r) async {
  return new v0.CalculationResponse(r.a + r.b);
}

Future<v0.CalculationResponse> calcSubtractHandler(
    v0.CalculationRequest r) async {
  return new v0.CalculationResponse(r.a - r.b);
}

Future<v0.CalculationResponse> calcMultiplyHandler(
    v0.CalculationRequest r) async {
  return new v0.CalculationResponse(r.a * r.b);
}

Future<v0.CalculationResponse> calcDivideHandler(
    v0.CalculationRequest r) async {
  // Delay by 1 second to run through future callbacks in Responses.forEach.
  return new Future<OmnibusResponse>.delayed(
      new Duration(seconds: 1), () => new v0.CalculationResponse(r.a / r.b));
}
